function [t, h, f, H] = plc_impuls( itype, fs, Nsamples )
%PLC_IMPULS 
% 
%   Function generates impulse response of the PLC channel according to samples number 
%   NSAMPLES, sampling frequency FS and impulsive responses types.
% 
%   One cane choose 13 Access type impulsive ITYPE:
%   -----------------------------------
%   ITYPE
%   ------------------------------------# OUTDOOR channels
%   11   - class 150 m, good            # 
%   12   - class 150 m, medium          #
%   13   - class 150 m, bad             #
%   ------------------------------------#
%   21   - class 250 m, good            #
%   22   - class 250 m, medium          #
%   ------------------------------------#
%   31   - class 350 m, good            #
%   32   - class 350 m, medium          #
%   33   - class 350 m, bad             #
%   ------------------------------------#
%   41   - class with one branch        #

%   ------------------------------------@ INDOOR channels
%   51   - class 1                      @ 
%   52   - class 2                      @
%   53   - class 3                      @
%   54   - class 4                      @
%   ------------------------------------@
%  
%   Examples:
%   [t, h, f, H] = plc_impuls(51,5e+7,256); plot(t,h), pause, plot(f,H)
%
%   Reference:
%   OPERA "THEORETICAL POSTULATION OF PLC CHANNEL MODEL"
%   www.ist-opera.org
%
%   $Revision: 0.1$ $Data: 29.12.2007$

if nargin == 2    % Nsamples are not specified
   switch  itype 
      case 11, tmax = 4e-6;
      case 12, tmax = 12e-6;
      case 13, tmax = 12e-6;
      case 21, tmax = 20e-6;
      case 22, tmax = 12e-6;
      case 31, tmax = 6e-6;
      case 32, tmax = 15e-6;
      case 33, tmax = 8e-6;
      case 41, tmax = 8e-6;
      case 51, tmax = 0.8e-6;  
      case 52, tmax = 1.2e-6;  
      case 53, tmax = 2e-6;  
      case 54, tmax = 2.5e-6;
      otherwise, error('Unknown itype. Please read help.');
   end
   if (1/fs) < tmax
      t = 0 : 1/fs : tmax;
   else
      error('Frequencies must be greater than 2e+5 Hz')
   end
   Nsamples = length(t);

else
   if nargin == 3
      n = 0:Nsamples-1;           
      t = n./fs;          % Time axis
   else
      error('Inappropriate number of parameters');
   end
end      

Nf = 1000;          % Number of samples H(f)

vp = 1.53e+8;       % Signa propagation speed
a0 = 0;             % Parameters responsible for suppresion
k = 1;

% Impulse response types switching

switch itype
   
    case 11,                       % Nr 1: 150 m, good =============================
        a1 = 1.65e-9; 
        g  = [0.09, -0.012, 0.012, -0.012 0.022].*1e+2;
        dm = [100, 130, 160, 190, 300];
        
    case 12,                       % Nr 2: 150 m, medium
        a1 = 2.8e-9;                    
        g  = [-0.15, 0.165, 0.032, -0.014, -0.035, -0.035, -0.03, 0.015, ... 
               0.022, 0.04, 0.02, -0.015, 0.0865, -0.062, -0.083, 0.05, -0.035].*1e+2;
        dm = [150.8, 152.3, 172, 210.4, 230, 258, 294, 370, 400, 435, 468, ...
              494, 534, 581, 632, 1070, 1224];
        
    case 13,                       % Nr 3: 150 m, bad
        a1 = 2.5e-9; 
        g  = [1.832, 0.516, 0.765, -1.031, -0.8, -0.711, 0.676, -0.676, ... 
              1.263, -0.622, 1.156, -0.978, 0.747, -1.049, 0.871]; %.*1e+2;
        dm = [113.2, 90.1, 101.8, 143, 148, 200, 322, 261, 411, 490, 567, ...
              740, 960, 1130, 1250];
        
    case 21,                       % Nr 4: 250 m, good ============================                        
        a1 = 5e-9;
        g = [-0.007, 0.034, -0.03, -0.031, -0.013, -0.015, 0.053, -0.022, ...
              0.07, -0.04, -0.052, -0.04, 0.088, -0.053].*1e+2;
        dm = [85, 103, 148, 195, 245, 315, 376, 438, 476, 530, 660, 800, ...
              1015, 1450 ];
        
    case 22,                       % Nr 5: 250 m, medium
        a1 = 4.5e-9;
        g = [3.02, 4.45, -1.78, -2.13, 5.87, -6.58, 3.2, -3.56, 0.89, ...
               -2.67, 2.67, -2.67];
        dm = [211.5, 228, 243, 254, 278, 306, 330, 360, 390, 420, 540, 740];
        
    case 31,                       % Nr 6: 350 m, good ============================
        a0 = 8e-3; a1 = 3.5e-9;
        g = [0.26, 0.05, -0.3, 0.25, -0.35]*1e+2;
        dm = [300, 350, 370, 450, 510];
        
    case 32,                       % Nr 7: 350 m, medium
        a1 = 9e-9;
        g = [0.00039, -0.0045, -0.0062, 0.0281, -0.0169, -0.0028, -0.0056, ...
             0.0051, 0.0112, -0.0141, 0.1125, -0.1687, 0.0675]*1e+2;
        dm = [40, 68, 86, 129, 185, 237, 235, 230, 450, 560, 830, 895, 1000];
        
    case 33,                       % Nr. 8: 350 m, bad
        a1 = 1.1e-8;
        g  = [-0.0039, 0.0156, -0.034, 0.0715, -0.122, 0.076]*1e+2;
        dm = [78, 126, 191, 256, 306, 330];
        
    case 41,                       % Nr 9: one branch ============================
        a1 = 8.8e-10;
        g  = [0.064, 0.038, -0.015, 0.005, -0.002, -0.003, -0.005, -0.006, -0.005, 0.002, -0.004]*1e+2;
        dm = [200, 222.4, 244.8, 267.5, 290, 350, 428, 440, 452, 630, 680 ];
        
    case 51,                       % Nr 10: indoor 1 ============================    
        ti  = [ 0.2  0.36  0.4  0.63  0.68 ]*1e-6;
        hti = [ 0.05  -0.0281  0.0158  0.0089  -0.0050 ];
        
    case 52,                       % Nr 11: indoor 2
        ti  = [ 0.2  0.34  0.4  0.54  0.63  0.72  0.89  1.01  1.11  1.15 ]*1e-6;
        hti = [ 0.01  -0.0084  -0.0070  0.0058  -0.0049  0.0041  -0.0034  -0.0029 -0.0025  0.0020 ];

    case 53,                       % Nr 12: indoor 3
        ti  = [ 0.5  0.65  0.7  0.84  0.93  1.01  1.12  1.20  1.31  1.51  1.60  1.70  1.82  1.86  1.96 ]*1e-6;
        hti = [ 3  -2.55  -2.16  1.83  -1.55  -1.32  -1.12  0.95  -0.8  -0.68  0.58  -0.49  -0.42  0.35  0.3 ]*1e-3;
       
    case 54,                       % Nr 13: indoor 4
        ti  = [ 0.5  0.65  0.75  0.84  0.91  1.06  1.15  1.24  1.30  1.45  1.57  1.65  1.73  1.89  1.98  2.11  2.21  2.33  2.42  2.46 ]*1e-6;
        hti = [ 0.5  -0.459  -0.422  -0.388  0.356  0.327  0.301  0.276  -0.254  -0.233  0.214  -0.197  0.181  0.166  -0.153  0.140  -0.129  -0.119  0.109  0.1 ]*1e-3;
       
    otherwise
    error('Unknown itype. Please read help.');
    
end

if( itype < 50 ) % OUTDOOR IMPULSE RESPONSES ####################################
        
    % ------------------------------------------------------------------------
    % Impulse response
    h = zeros(1,Nsamples);      % Prepare place for the imp. resp.
    hk = zeros(1,Nsamples);

    for n = 1:length(g),
    for m = 1:Nsamples,
   	  h(m) = sum( g(n).*exp(-a0.*dm(n))*a1.*dm(n)/((a1.*dm(n)/2/pi).^2+(t(m)-dm(n)./vp).^2)/(2*pi*pi) );
    end
	    hk = hk + h;
    end
    h = 0.01*hk/fs;

    % ------------------------------------------------------------------------
    % Frequency response
    f = linspace(0,fs/2,Nf);

    H = zeros(1,Nf);
    for m = 1 : Nf,
        H(m) = sum((g.*exp(-(a0+a1.*f(m).^k).*dm)) .* exp(-j*2*pi*f(m).*dm./vp));
    end
    H = 0.01*abs(H);

else          % INDOOR IMPULSE RESPONSES ######################################
   
     h = zeros( 1, Nsamples );
     h( round(ti*fs)+1 ) = hti;
     %stem( t(1:150), h(1:150) ); pause
     H = abs( fft(h,Nf) );
     f = 0 : fs/Nf : fs/2;
     H = H(1:Nf/2+1);
     %plot(f, 20*log10(H) ); pause
end     
    
